#include "AddonManagerBinding.h"
#include "ChromeUtilsBinding.h"
#include "DOMErrorBinding.h"
#include "DOMRequestBinding.h"
#include "DOMStringListBinding.h"
#include "DominatorTreeBinding.h"
#include "ErrorEventBinding.h"
#include "EventBinding.h"
#include "EventTargetBinding.h"
#include "FileReaderBinding.h"
#include "FlyWebDiscoveryManagerBinding.h"
#include "HeapSnapshotBinding.h"
#include "IDBCursorBinding.h"
#include "IDBDatabaseBinding.h"
#include "IDBFactoryBinding.h"
#include "IDBFileHandleBinding.h"
#include "IDBFileRequestBinding.h"
#include "IDBIndexBinding.h"
#include "IDBKeyRangeBinding.h"
#include "IDBMutableFileBinding.h"
#include "IDBObjectStoreBinding.h"
#include "IDBOpenDBRequestBinding.h"
#include "IDBRequestBinding.h"
#include "IDBTransactionBinding.h"
#include "IDBVersionChangeEventBinding.h"
#include "MessageEventBinding.h"
#include "MessagePortBinding.h"
#include "PromiseDebuggingBinding.h"
#include "TCPServerSocketBinding.h"
#include "TCPServerSocketEventBinding.h"
#include "TCPSocketBinding.h"
#include "TCPSocketErrorEventBinding.h"
#include "TCPSocketEventBinding.h"
#include "TextDecoderBinding.h"
#include "TextEncoderBinding.h"
#include "ThreadSafeChromeUtilsBinding.h"
#include "URLSearchParamsBinding.h"
#include "WorkerBinding.h"
#include "js/Id.h"
#include "mozilla/dom/BindingUtils.h"
#include "nsThreadUtils.h"

namespace mozilla {
namespace dom {
bool
ResolveSystemBinding(JSContext* aCx, JS::Handle<JSObject*> aObj, JS::Handle<jsid> aId, bool* aResolvedp)
{
  MOZ_ASSERT(NS_IsMainThread());

  static jsid sAddonManagerPermissions_id;
  static jsid sChromeUtils_id;
  static jsid sChromeWorker_id;
  static jsid sDOMError_id;
  static jsid sDOMRequest_id;
  static jsid sDOMStringList_id;
  static jsid sDominatorTree_id;
  static jsid sErrorEvent_id;
  static jsid sEvent_id;
  static jsid sEventTarget_id;
  static jsid sFileReader_id;
  static jsid sFlyWebDiscoveryManager_id;
  static jsid sHeapSnapshot_id;
  static jsid sIDBCursor_id;
  static jsid sIDBCursorWithValue_id;
  static jsid sIDBDatabase_id;
  static jsid sIDBFactory_id;
  static jsid sIDBFileHandle_id;
  static jsid sIDBFileRequest_id;
  static jsid sIDBIndex_id;
  static jsid sIDBKeyRange_id;
  static jsid sIDBLocaleAwareKeyRange_id;
  static jsid sIDBMutableFile_id;
  static jsid sIDBObjectStore_id;
  static jsid sIDBOpenDBRequest_id;
  static jsid sIDBRequest_id;
  static jsid sIDBTransaction_id;
  static jsid sIDBVersionChangeEvent_id;
  static jsid sMessageEvent_id;
  static jsid sMessagePort_id;
  static jsid sPromiseDebugging_id;
  static jsid sTCPServerSocket_id;
  static jsid sTCPServerSocketEvent_id;
  static jsid sTCPSocket_id;
  static jsid sTCPSocketErrorEvent_id;
  static jsid sTCPSocketEvent_id;
  static jsid sTextDecoder_id;
  static jsid sTextEncoder_id;
  static jsid sThreadSafeChromeUtils_id;
  static jsid sURLSearchParams_id;
  static jsid sWorker_id;

  static bool idsInited = false;
  if (!idsInited) {
    if (!AtomizeAndPinJSString(aCx, sAddonManagerPermissions_id, "AddonManagerPermissions")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sChromeUtils_id, "ChromeUtils")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sChromeWorker_id, "ChromeWorker")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sDOMError_id, "DOMError")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sDOMRequest_id, "DOMRequest")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sDOMStringList_id, "DOMStringList")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sDominatorTree_id, "DominatorTree")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sErrorEvent_id, "ErrorEvent")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sEvent_id, "Event")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sEventTarget_id, "EventTarget")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sFileReader_id, "FileReader")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sFlyWebDiscoveryManager_id, "FlyWebDiscoveryManager")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sHeapSnapshot_id, "HeapSnapshot")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sIDBCursor_id, "IDBCursor")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sIDBCursorWithValue_id, "IDBCursorWithValue")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sIDBDatabase_id, "IDBDatabase")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sIDBFactory_id, "IDBFactory")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sIDBFileHandle_id, "IDBFileHandle")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sIDBFileRequest_id, "IDBFileRequest")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sIDBIndex_id, "IDBIndex")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sIDBKeyRange_id, "IDBKeyRange")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sIDBLocaleAwareKeyRange_id, "IDBLocaleAwareKeyRange")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sIDBMutableFile_id, "IDBMutableFile")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sIDBObjectStore_id, "IDBObjectStore")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sIDBOpenDBRequest_id, "IDBOpenDBRequest")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sIDBRequest_id, "IDBRequest")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sIDBTransaction_id, "IDBTransaction")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sIDBVersionChangeEvent_id, "IDBVersionChangeEvent")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sMessageEvent_id, "MessageEvent")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sMessagePort_id, "MessagePort")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sPromiseDebugging_id, "PromiseDebugging")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sTCPServerSocket_id, "TCPServerSocket")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sTCPServerSocketEvent_id, "TCPServerSocketEvent")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sTCPSocket_id, "TCPSocket")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sTCPSocketErrorEvent_id, "TCPSocketErrorEvent")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sTCPSocketEvent_id, "TCPSocketEvent")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sTextDecoder_id, "TextDecoder")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sTextEncoder_id, "TextEncoder")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sThreadSafeChromeUtils_id, "ThreadSafeChromeUtils")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sURLSearchParams_id, "URLSearchParams")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sWorker_id, "Worker")) {
      return false;
    }

    idsInited = true;
  }

  if ((JSID_IS_VOID(aId) || aId == sAddonManagerPermissions_id) && AddonManagerPermissionsBinding::ConstructorEnabled(aCx, aObj)) {
    if (!AddonManagerPermissionsBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if ((JSID_IS_VOID(aId) || aId == sChromeUtils_id) && ChromeUtilsBinding::ConstructorEnabled(aCx, aObj)) {
    if (!ChromeUtilsBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if ((JSID_IS_VOID(aId) || aId == sChromeWorker_id) && ChromeWorkerBinding::ConstructorEnabled(aCx, aObj)) {
    if (!ChromeWorkerBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sDOMError_id) {
    if (!DOMErrorBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sDOMRequest_id) {
    if (!DOMRequestBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sDOMStringList_id) {
    if (!DOMStringListBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if ((JSID_IS_VOID(aId) || aId == sDominatorTree_id) && DominatorTreeBinding::ConstructorEnabled(aCx, aObj)) {
    if (!DominatorTreeBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sErrorEvent_id) {
    if (!ErrorEventBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sEvent_id) {
    if (!EventBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sEventTarget_id) {
    if (!EventTargetBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sFileReader_id) {
    if (!FileReaderBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if ((JSID_IS_VOID(aId) || aId == sFlyWebDiscoveryManager_id) && FlyWebDiscoveryManagerBinding::ConstructorEnabled(aCx, aObj)) {
    if (!FlyWebDiscoveryManagerBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if ((JSID_IS_VOID(aId) || aId == sHeapSnapshot_id) && HeapSnapshotBinding::ConstructorEnabled(aCx, aObj)) {
    if (!HeapSnapshotBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sIDBCursor_id) {
    if (!IDBCursorBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sIDBCursorWithValue_id) {
    if (!IDBCursorWithValueBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sIDBDatabase_id) {
    if (!IDBDatabaseBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sIDBFactory_id) {
    if (!IDBFactoryBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sIDBFileHandle_id) {
    if (!IDBFileHandleBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sIDBFileRequest_id) {
    if (!IDBFileRequestBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sIDBIndex_id) {
    if (!IDBIndexBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sIDBKeyRange_id) {
    if (!IDBKeyRangeBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if ((JSID_IS_VOID(aId) || aId == sIDBLocaleAwareKeyRange_id) && IDBLocaleAwareKeyRangeBinding::ConstructorEnabled(aCx, aObj)) {
    if (!IDBLocaleAwareKeyRangeBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sIDBMutableFile_id) {
    if (!IDBMutableFileBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sIDBObjectStore_id) {
    if (!IDBObjectStoreBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sIDBOpenDBRequest_id) {
    if (!IDBOpenDBRequestBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sIDBRequest_id) {
    if (!IDBRequestBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sIDBTransaction_id) {
    if (!IDBTransactionBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sIDBVersionChangeEvent_id) {
    if (!IDBVersionChangeEventBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sMessageEvent_id) {
    if (!MessageEventBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sMessagePort_id) {
    if (!MessagePortBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if ((JSID_IS_VOID(aId) || aId == sPromiseDebugging_id) && PromiseDebuggingBinding::ConstructorEnabled(aCx, aObj)) {
    if (!PromiseDebuggingBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if ((JSID_IS_VOID(aId) || aId == sTCPServerSocket_id) && TCPServerSocketBinding::ConstructorEnabled(aCx, aObj)) {
    if (!TCPServerSocketBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if ((JSID_IS_VOID(aId) || aId == sTCPServerSocketEvent_id) && TCPServerSocketEventBinding::ConstructorEnabled(aCx, aObj)) {
    if (!TCPServerSocketEventBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if ((JSID_IS_VOID(aId) || aId == sTCPSocket_id) && TCPSocketBinding::ConstructorEnabled(aCx, aObj)) {
    if (!TCPSocketBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if ((JSID_IS_VOID(aId) || aId == sTCPSocketErrorEvent_id) && TCPSocketErrorEventBinding::ConstructorEnabled(aCx, aObj)) {
    if (!TCPSocketErrorEventBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if ((JSID_IS_VOID(aId) || aId == sTCPSocketEvent_id) && TCPSocketEventBinding::ConstructorEnabled(aCx, aObj)) {
    if (!TCPSocketEventBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sTextDecoder_id) {
    if (!TextDecoderBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sTextEncoder_id) {
    if (!TextEncoderBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if ((JSID_IS_VOID(aId) || aId == sThreadSafeChromeUtils_id) && ThreadSafeChromeUtilsBinding::ConstructorEnabled(aCx, aObj)) {
    if (!ThreadSafeChromeUtilsBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sURLSearchParams_id) {
    if (!URLSearchParamsBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  if ((JSID_IS_VOID(aId) || aId == sWorker_id) && WorkerBinding::ConstructorEnabled(aCx, aObj)) {
    if (!WorkerBinding::GetConstructorObject(aCx)) {
      return false;
    }
    *aResolvedp = true;
  }

  return true;
}

} // namespace dom
} // namespace mozilla

